/* libfir/src/libfir/200_common_filters.h
 *
 *  Copyright: ©2014, Laurence Withers.
 *  Author: Laurence Withers <l@lwithers.me.uk>
 *  License: GPLv3
 */



/*! \defgroup libfir_common Common filter types

\ingroup libfir

Routines in this section instantiate %FIR filters using algorithms to build
commonly-used types of filter, such as brickwall low-pass etc.

*/
/*!@{*/



/*! \brief Brickwall low-pass linear phase: sinc filter.

\param npoints Number of points required (≥1).
\param corner Corner frequency as fraction of sample rate.
\param gain Gain of filter.
\returns Newly-allocated %FIR filter.

Builds a truncated-sinc filter which has a low-pass, brickwall effect (cutoff
at \a corner, which is a fraction of sampling rate, i.e. 0 &lt; \a corner &lt;
0.5).

The number of points \a npoints is a trade-off between accuracy of filter (an
ideal sinc filter would have an infinite number of points) and computation time.
\a npoints may be odd or even; symmetry will be exploited in either case.

The DC gain of the filter is normalised to \a gain, which is often 1.0.

*/
struct fir_filter_t* fir_sinc_lowpass(int npoints, double corner, double gain);



/*!@}*/
/* options for text editors
vim: expandtab:ts=4:sw=4:syntax=ch.doxygen
*/
